#define SWAP(a, b) { int c = a; a = b; b = c; }

#ifdef ESP8266

#define VAL_MIN   16
#define VAL_MAX   80
#define PWM_MIN   40
#define PWM_MAX   90

void AnalogWrite(int pin1, int pwm, int pwmCH) {
  analogWrite(pin1, pwm);
}

void AnalogWriteD(int pin1, int pin2, int pwm, int pwmCH) {
  if (pwm < 0) {
    pwm = (0 - pwm);
    SWAP(pin1, pin2);
  }
  pwm = ((pwm < VAL_MIN) ? 0 : map(pwm, VAL_MIN, VAL_MAX, (1024 * PWM_MIN / 100), (1024 * PWM_MAX / 100)));
  analogWrite(pin2, 0);
  if (pwm >= (1024 * PWM_MAX / 100))
    digitalWrite(pin1, HIGH);
  else
    analogWrite(pin1, pwm);
}

#else // for ESP32

#define VAL_MIN   16
#define VAL_MAX   80
#define PWM_MIN   30
#define PWM_MAX   95

#define PWM_BITW  10
#define PWM_FREQ  500
const int pwmFreq[] = { PWM_FREQ, PWM_FREQ, PWM_FREQ, PWM_FREQ, PWM_FREQ, PWM_FREQ, -1 };

void AnalogWrite(int pin1, int pwm, int pwmCH) {
  AnalogWriteD(pin1, 0, pwm, pwmCH);
}

void AnalogWriteD(int pin1, int pin2, int pwm, int pwmCH) {
  static int sw = -1, pwmPin[4] = { -1, -1, -1, -1 };

  if (sw == -1) {
    for (sw = 0; (pwmFreq[sw] != -1); sw++) {
      ledcSetup(sw, pwmFreq[sw], PWM_BITW);
    }
  }

  if (pwm < 0) {
    pwm = (0 - pwm);
    SWAP(pin1, pin2);
  }
  pwm = ((pwm < VAL_MIN) ? 0 : map(pwm, VAL_MIN, VAL_MAX, (1024 * PWM_MIN / 100), (1024 * PWM_MAX / 100)));

  if (pwmPin[pwmCH] != pin1) {
    if (pin2 != 0) {
      ledcAttachPin(pin2, (pwmCH + 4));
      ledcWrite((pwmCH + 4), 0);
    }
    ledcAttachPin((pwmPin[pwmCH] = pin1), pwmCH);
  }
  ledcWrite(pwmCH, ((pwm > 1023) ? 1023 : pwm));
}

#endif
