
#define uSSID    "ESPCar02"
#define uPASS    "ESP12345"

#ifdef ESP8266
//------------------
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
ESP8266WebServer server(80);

#define POWER_SW  14
#define LED_Light  2
#define LED_Hone  15
#define PWM_RF     5
#define PWM_RR    13
#define PWM_LF     4
#define PWM_LR    12

#else // for ESP32
//------------------
#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>
WebServer server(80);

#ifdef ARDUINO_M5Stack_ATOM
//------------------
#define LED_Light 23
#define LED_Hone  35
#define PWM_RF    25
#define PWM_RR    19
#define PWM_LF    22
#define PWM_LR    21

#else
//------------------
#define LED_Light 32
#define LED_Hone  33
#define PWM_RF    27
#define PWM_RR    14
#define PWM_LF    12
#define PWM_LR    13

#endif
#endif

#define PWM_CH0    0
#define PWM_CH1    1
#define PWM_CH2    2

#include "index_html2.h"

void handleNotFound() {
  String message = "File Not Found\n\n"
                   + server.uri() + " "
                   + ((server.method() == HTTP_GET) ? "GET" : "POST")
                   + "\nArguments: " + server.args() + "\n";
  for (int i = 0; i < server.args(); i++)
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  server.send(404, "text/plain", message);
}

int NoDat = 0, timeOut = 0;
int Val_C = 0;  // Horn

#ifdef POWER_SW
void powerCnt(int sw) {
  pinMode(POWER_SW, OUTPUT);
  if (sw) {
    digitalWrite(POWER_SW, HIGH);
    return;
  }
  for (;;) {
    digitalWrite(POWER_SW, LOW);
    noInterrupts();
  }
}
#else
#define powerCnt(n)
#endif

void handleRC() {
  NoDat = 0;
  int j = 0;
  for (int i = 0; i < server.args(); i++) {
    int Val_i = server.arg(i).toInt();
    if (Val_i) {
      Serial.print(server.argName(i) + "=" + server.arg(i) + ", ");
      j++;
    }
    switch (server.argName(i)[0]) {
      case 'F': digitalWrite(LED_Light, Val_i); break;
      case 'C': if (Val_C += Val_i) AnalogWrite(LED_Hone, 512, PWM_CH2); break;
      case 'R': AnalogWriteD(PWM_RF, PWM_RR, Val_i, PWM_CH0); break;
      case 'L': AnalogWriteD(PWM_LF, PWM_LR, Val_i, PWM_CH1); break;
    }
  }
  if (j) Serial.println(j);
  server.send(200, "text/plain", "\r\n\r\n\r\n");
}

void setup(void) {
  Serial.begin(115200);
  Serial.printf("\nStart\n");

  pinMode(LED_Light, OUTPUT); digitalWrite(LED_Light, 0);
  pinMode(LED_Hone , OUTPUT); digitalWrite(LED_Hone , 0);
  pinMode(PWM_LF   , OUTPUT); digitalWrite(PWM_LF   , 0);
  pinMode(PWM_LR   , OUTPUT); digitalWrite(PWM_LR   , 0);
  pinMode(PWM_RF   , OUTPUT); digitalWrite(PWM_RF   , 0);
  pinMode(PWM_RR   , OUTPUT); digitalWrite(PWM_RR   , 0);

  WiFi.softAP(uSSID, uPASS);
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("AP address: ");  Serial.println(myIP);

  server.on("/", []() { server.send(200, "text/html", index_html); });
  server.on("/rc", handleRC);
  server.onNotFound(handleNotFound);
  server.begin();
  Serial.println("HTTP server started");
}

void loop(void) {
  server.handleClient();
  if (Val_C && --Val_C == 0) AnalogWrite(LED_Hone, 0, PWM_CH2);
  if (NoDat > 50) {
    AnalogWriteD(PWM_RF, PWM_RR, 0, PWM_CH0);
    AnalogWriteD(PWM_LF, PWM_LR, 0, PWM_CH1);
    AnalogWrite (LED_Hone,  (Val_C = 0), PWM_CH2);
    digitalWrite(LED_Light, !digitalRead(LED_Light));
    NoDat = 0;
    if (++timeOut > 120) powerCnt(0);
  } else NoDat++;
  delay(10);
}
